
;    little halma      by martin jahn
;
;   for bmp-files is searched in the current dir


IncludeFile "init1.pb"

;#PI = 3.14159265
#speed = 7 ; speed of the ant (pixel per move)

Structure piece
	valid.b ; whether the place is part of board or not (1/0)
	full.b ;whether there is a piece or not (1/0)
EndStructure
Global Dim spot.piece(6,6) ; thats a map ( 7*7)
Global Dim undo_spot.piece(6,6)

Structure mouseStruct
	focus.b ; whether the mouse is over the gaming-board or not (1/0)     (not the window-focus!)
	x.b ; the place on the board (not a pixel value but array-index)
	y.b ; the place on the board (not a pixel value but array-index)
	direction.b ; takes up values of constants like #left
	moveIsValid.b ; whether the current direction is valid move or not (1/0)
EndStructure
Global mouse.mouseStruct

Structure antStruct
	x.w ; pixel-coordinate
	y.w ; pixel-coordinate
	direction.b
EndStructure
Global ant.antStruct

Global firstIsCropped.b ; whether the starting piece is cropped or not (1/0)
Global undo_firstIsCropped.b
Global status.s ; text of statusbar
Global undo_status.s
Global piecesLeft.w
firstIsCropped = 0
status = "click to crop!"

Declare setDefaultMap() ; creates the standard map
Declare drawMap()
Declare drawMouse()
Declare.f getAlpha(x.w, y.w)
Declare getMouseData()
Declare.b moveValidity()
Declare movePiece()
Declare drawAnt()
Declare	saveUndoData()
Declare	loadUndoData()
Declare.w getPossibleMoveAmount()


;- MAIN ##########################################################################################################
setDefaultMap()
saveUndoData() ; just to fill the undo-backups with valid ( but useless ) values, in case of the user clicks undo before moving/cropping a piece
Repeat
	Delay(80)
	getMouseData()
	EventID = WindowEvent() ; can't use WaitWindowEvent, because animations must go on
	If EventID = #PB_Event_Menu And EventMenu() = 1 ; MenuItem "undo"
		loadUndoData()
	EndIf
	If EventID = 513 ; left mouse button
		saveUndoData()
		If firstIsCropped = 0 And spot( mouse\x,mouse\y )\full
				spot( mouse\x,mouse\y )\full = 0
				firstIsCropped = 1
				
		ElseIf mouse\moveIsValid
			movePiece()
		EndIf
		status = Str( getPossibleMoveAmount() ) + " possible moves"
	EndIf
	ClearScreen( RGB(200,200,200) )
	StartDrawing(WindowOutput(0))
	drawMap()
	drawMouse()
	StopDrawing()
	FlipBuffers()
	StatusBarText(0,0,"   "+status)
	If getPossibleMoveAmount() = 0 And firstIsCropped = 1
		Delay(1000)
		MessageRequester("you finished", "no more moves possible. " +Str(piecesLeft) + " pieces left", 0)
		End
	EndIf
	Until EventID = #PB_Event_CloseWindow;Quit = 1
End ; end of MAIN #################################################################################################

Procedure setDefaultMap()
	For i=0 To 6
		For k=0 To 6
			If (i=0 Or i=1 Or i=5 Or i=6) And (k=0 Or k=1 Or k=5 Or k=6)
				spot(i,k)\valid = 0
			Else
				spot(i,k)\valid = 1
			EndIf
			spot(i,k)\full = spot(i,k)\valid
		Next
	Next
EndProcedure

Procedure drawMap()
	For i=0 To 6
		For k=0 To 6
			DisplaySprite(#bg, i*32, k*32)
			If spot(i,k)\valid
				DisplayTransparentSprite(#valid,i*32,k*32)
			EndIf
		Next
	Next
	drawAnt()
	piecesLeft = 0
	For i=0 To 6
		For k=0 To 6
			If spot(i,k)\full
				DisplayTransparentSprite(#full,i*32 + Random(1)*-1*Random(100)/95, k*32 + Random(1)*-1*Random(100)/95)
				piecesLeft = piecesLeft + 1
			EndIf
		Next
	Next
EndProcedure

Procedure drawMouse()
	If mouse\focus
		If spot(mouse\x,mouse\y)\full
			DisplayTransparentSprite(#full2,mouse\x*32+Random(4)-2,mouse\y*32+Random(4)-2)
			If firstIsCropped = 0
				DisplayTransparentSprite(#cross,mouse\x*32,mouse\y*32)
			ElseIf mouse\moveIsValid
				DisplayTransparentSprite( mouse\direction, mouse\x*32-32, mouse\y*32-32) ; minus 32, cause its an 96 pixel graphic (3*32)
			EndIf
		EndIf
	EndIf
EndProcedure

Procedure.f getAlpha(x.w, y.w)
	result.f
	If x=0
		x=1 ; to prevent division by zero
	EndIf
	If 		x>0 And y<=0
		result =            ATan(-y/x) * 180.0/#PI
	ElseIf x<=0 And y<0
		result = 180 - (ATan(-y/-x) * 180.0/#PI)
	ElseIf x<=0 And y>=0
		result = 180 + (ATan(y/-x) * 180.0/#PI)
	ElseIf x>0   And y>0
		result = 360 - ATan(y/x) * 180.0/#PI
	EndIf
	ProcedureReturn result
EndProcedure

Procedure getMouseData()
	If WindowMouseX(0) > #win_border And WindowMouseY(0) > #win_border And WindowMouseX(0) < #screen_width+#win_border And WindowMouseY(0) < #screen_height+#win_border
		mouse\focus=1
		mouse\x = (WindowMouseX(0)-#win_border)/32
		mouse\y = (WindowMouseY(0)-#win_border)/32
		
		alpha = getAlpha(   ((WindowMouseX(0)-#win_border)%32)-16,   ((WindowMouseY(0)-#win_border)%32)-16   )
		If alpha > 135 And alpha <= 225
			mouse\direction=#left
		ElseIf alpha > 45 And alpha <= 135
			mouse\direction=#up
		ElseIf alpha <= 45 Or alpha > 315
			mouse\direction=#right
		ElseIf alpha > 225 And alpha <= 315
			mouse\direction=#down
		EndIf
		mouse\moveIsValid = moveValidity()
	Else
		mouse\focus=0
	EndIf
EndProcedure

Procedure.b moveValidity()
	Select mouse\direction
		Case #left
			If mouse\x >= 2 ; to prevent "error : Array index out of bounds" in the following
				If spot(mouse\x, mouse\y)\full And spot( mouse\x - 1, mouse\y )\full And spot( mouse\x - 2, mouse\y )\valid And spot( mouse\x - 2, mouse\y )\full=0
					ProcedureReturn 1
				EndIf
			EndIf
		Case #up
			If mouse\y >= 2
				If spot(mouse\x, mouse\y)\full And spot( mouse\x, mouse\y - 1 )\full And spot( mouse\x, mouse\y - 2 )\valid And spot( mouse\x, mouse\y - 2 )\full=0
					ProcedureReturn 1
				EndIf
			EndIf
		Case #right
			If mouse\x <= 4
				If spot(mouse\x, mouse\y)\full And spot( mouse\x + 1, mouse\y )\full And spot( mouse\x + 2, mouse\y )\valid And spot( mouse\x + 2, mouse\y )\full=0
					ProcedureReturn 1
				EndIf
			EndIf
		Case #down
			If mouse\y <= 4
				If spot(mouse\x, mouse\y)\full And spot( mouse\x, mouse\y + 1 )\full And spot( mouse\x, mouse\y + 2 )\valid And spot( mouse\x, mouse\y + 2 )\full=0
					ProcedureReturn 1
				EndIf
			EndIf
		Default
			ProcedureReturn 0
	EndSelect
EndProcedure

Procedure movePiece()
	Select mouse\direction
		Case #left
			spot(mouse\x, mouse\y)\full = 0
			spot(mouse\x - 1, mouse\y)\full = 0
			spot(mouse\x - 2, mouse\y)\full = 1
		Case #up
			spot(mouse\x, mouse\y)\full = 0
			spot(mouse\x, mouse\y - 1)\full = 0
			spot(mouse\x, mouse\y - 2)\full = 1
		Case #right
			spot(mouse\x, mouse\y)\full = 0
			spot(mouse\x + 1, mouse\y)\full = 0
			spot(mouse\x + 2, mouse\y)\full = 1
		Case #down
			spot(mouse\x, mouse\y)\full = 0
			spot(mouse\x, mouse\y + 1)\full = 0
			spot(mouse\x, mouse\y + 2)\full = 1
	EndSelect
EndProcedure

Procedure drawAnt()
	DisplayTransparentSprite( ant\direction + #ant_first, ant\x,ant\y)
	ant\direction = ant\direction + Random(2)-1
	If ant\direction = -1
		ant\direction = 7
	ElseIf ant\direction = 8
		ant\direction = 0
	EndIf
	
	If ant\direction=0 Or ant\direction=1 Or ant\direction=2
		ant\x = ant\x + #speed
	EndIf
	If  ant\direction=4 Or ant\direction=5 Or ant\direction=6
		ant\x = ant\x - #speed
	EndIf
	If  ant\direction=6 Or ant\direction=7 Or ant\direction=0
		ant\y = ant\y + #speed
	EndIf
	If  ant\direction=2 Or  ant\direction=3 Or  ant\direction=4
		ant\y = ant\y - #speed
	EndIf
	

	If ant\x < -16
		ant\x = #screen_width
	ElseIf ant\x > #screen_width
		ant\x = -16
	EndIf
	
	If ant\y < -16
		ant\y = #screen_width
	ElseIf ant\y > #screen_width
		ant\y = -16
	EndIf
EndProcedure

Procedure saveUndoData()
	For i=0 To 6
		For k=0 To 6
			undo_spot(i,k)\valid = spot(i,k)\valid
			undo_spot(i,k)\full = spot(i,k)\full
			undo_firstIsCropped = firstIsCropped
			undo_status = status
		Next
	Next
EndProcedure

Procedure loadUndoData()
	For i=0 To 6
		For k=0 To 6
			spot(i,k)\valid = undo_spot(i,k)\valid
			spot(i,k)\full = undo_spot(i,k)\full
			firstIsCropped = undo_firstIsCropped
			status = undo_status
		Next
	Next
EndProcedure

 Procedure.w getPossibleMoveAmount()
 	result.w = 0
	For i=0 To 6
		For k=0 To 6
			For eachDirection = #left To #down ; left --> up --> right --> down
				mouse\x = i
				mouse\y = k
				mouse\direction = eachDirection
				result = result + MoveValidity()
			Next
		Next
	Next
	ProcedureReturn result
 EndProcedure
; IDE Options = PureBasic v4.00 (Windows - x86)
; CursorPosition = 5
; FirstLine = 1
; Folding = --
; Markers = 24
; Executable = C:\Dokumente und Einstellungen\mo\Desktop\hlm.exe